@@ -1,7 +1,7 @@
 /*
  *  config_file.c - manages config file access.
  *	part of galculator
- *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2007 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  
 /* to add a new preference, add it to 
 	- s_preferences in config_file.h
+	- #define DEFAULTS in config_file.h
 	- to config_file_get_default_prefs
 	- to struct prefs_list (increase array size!)
 */
@@ -39,37 +40,49 @@
 #include "general_functions.h"
 
 s_preferences prefs;
-static s_prefs_entry prefs_list[41] = {
-	{"display_bkg_color", &(prefs.bkg_color), STRING, "prefs_bkg_color", set_button_color},
-	{"display_result_font", &(prefs.result_font), STRING, "prefs_result_font", set_button_label},
-	{"display_result_color", &(prefs.result_color), STRING, "prefs_result_color", set_button_color},
-	{"display_stack_font", &(prefs.stack_font), STRING, "prefs_stack_font", set_button_label},
-	{"display_stack_color", &(prefs.stack_color), STRING, "prefs_stack_color", set_button_color},
-	{"display_module_font", &(prefs.mod_font), STRING, "prefs_mod_font", set_button_label},
-	{"display_module_active_color", &(prefs.act_mod_color), STRING, "prefs_act_mod_color", set_button_color},
-	{"display_module_inactive_color", &(prefs.inact_mod_color), STRING, "prefs_inact_mod_color", set_button_color},
+static s_prefs_entry prefs_list[53] = {
+	{"display_bkg_color", &(prefs.bkg_color), STRING, "prefs_bkg_color_button", set_button_color},
+	{"display_result_font", &(prefs.result_font), STRING, "prefs_result_font", set_button_font},
+	{"display_result_color", &(prefs.result_color), STRING, "prefs_result_color_button", set_button_color},
+	{"display_stack_font", &(prefs.stack_font), STRING, "prefs_stack_font", set_button_font},
+	{"display_stack_color", &(prefs.stack_color), STRING, "prefs_stack_color_button", set_button_color},
+	{"display_module_font", &(prefs.mod_font), STRING, "prefs_mod_font", set_button_font},
+	{"display_module_active_color", &(prefs.act_mod_color), STRING, "prefs_act_mod_color_button", set_button_color},
+	{"display_module_inactive_color", &(prefs.inact_mod_color), STRING, "prefs_inact_mod_color_button", set_button_color},
 	{"display_module_number", &(prefs.vis_number), BOOLEAN, "prefs_vis_number", set_checkbutton},
-/*10.*/	{"display_module_angle", &(prefs.vis_angle), BOOLEAN, "prefs_vis_angle", set_checkbutton},
+/*10*/	{"display_module_angle", &(prefs.vis_angle), BOOLEAN, "prefs_vis_angle", set_checkbutton},
 	{"display_module_notation", &(prefs.vis_notation), BOOLEAN, "prefs_vis_notation", set_checkbutton},
 	{"display_module_arith", &(prefs.vis_arith), BOOLEAN, "prefs_vis_arith", set_checkbutton},
 	{"display_module_open", &(prefs.vis_bracket), BOOLEAN, "prefs_vis_bracket", set_checkbutton},
 	{"custom_button_font", &(prefs.custom_button_font), BOOLEAN, "prefs_custom_button_font", set_checkbutton},
-	{"button_font", &(prefs.button_font), STRING, "prefs_button_font", set_button_label},
+	{"button_font", &(prefs.button_font), STRING, "prefs_button_font", set_button_font},
 	{"button_width", &(prefs.button_width), INTEGER, "prefs_button_width", set_spinbutton},
 	{"button_height", &(prefs.button_height), INTEGER, "prefs_button_height", set_spinbutton},
 	{"function_button_group", &(prefs.vis_funcs), BOOLEAN, NULL, NULL},
 	{"dispctrl_button_group", &(prefs.vis_dispctrl), BOOLEAN, NULL, NULL},
-/*20.*/	{"logic_button_group", &(prefs.vis_logic), BOOLEAN, NULL, NULL},
+/*20*/	{"logic_button_group", &(prefs.vis_logic), BOOLEAN, NULL, NULL},
 	{"standard_button_group", &(prefs.vis_standard), BOOLEAN, NULL, NULL},
 	{"mode", &(prefs.mode), INTEGER, NULL, NULL},
+	{"dec_sep", &(prefs.dec_sep), BOOLEAN, "prefs_dec_sep", set_checkbutton},
+	{"dec_sep_length", &(prefs.dec_sep_length), INTEGER, "prefs_dec_sep_length", set_spinbutton},
+	{"dec_sep_char", &(prefs.dec_sep_char), STRING, "prefs_dec_sep_char", set_entry},
 	{"hex_bits", &(prefs.hex_bits), INTEGER, "prefs_hex_bits", set_spinbutton},
 	{"hex_signed", &(prefs.hex_signed), BOOLEAN, "prefs_hex_signed", set_checkbutton},
+	{"hex_sep", &(prefs.hex_sep), BOOLEAN, "prefs_hex_sep", set_checkbutton},
+	{"hex_sep_length", &(prefs.hex_sep_length), INTEGER, "prefs_hex_sep_length", set_spinbutton},
+/*30*/	{"hex_sep_char", &(prefs.hex_sep_char), STRING, "prefs_hex_sep_char", set_entry},
 	{"oct_bits", &(prefs.oct_bits), INTEGER, "prefs_oct_bits", set_spinbutton},
 	{"oct_signed", &(prefs.oct_signed), BOOLEAN, "prefs_oct_signed", set_checkbutton},
+	{"oct_sep", &(prefs.oct_sep), BOOLEAN, "prefs_oct_sep", set_checkbutton},
+	{"oct_sep_length", &(prefs.oct_sep_length), INTEGER, "prefs_oct_sep_length", set_spinbutton},
+	{"oct_sep_char", &(prefs.oct_sep_char), STRING, "prefs_oct_sep_char", set_entry},
 	{"bin_bits", &(prefs.bin_bits), INTEGER, "prefs_bin_bits", set_spinbutton},
 	{"bin_signed", &(prefs.bin_signed), BOOLEAN, "prefs_bin_signed", set_checkbutton},
 	{"bin_fixed", &(prefs.bin_fixed), BOOLEAN, "prefs_bin_fixed", set_checkbutton},
-/*30.*/	{"bin_length", &(prefs.bin_length), INTEGER, "prefs_bin_length", set_spinbutton},
+	{"bin_length", &(prefs.bin_length), INTEGER, "prefs_bin_length", set_spinbutton},
+/*40*/	{"bin_sep", &(prefs.bin_sep), BOOLEAN, "prefs_bin_sep", set_checkbutton},
+	{"bin_sep_length", &(prefs.bin_sep_length), INTEGER, "prefs_bin_sep_length", set_spinbutton},
+	{"bin_sep_char", &(prefs.bin_sep_char), STRING, "prefs_bin_sep_char", set_entry},
 	{"default_number_base", &(prefs.def_number), INTEGER, NULL, NULL},
 	{"default_angle_base", &(prefs.def_angle), INTEGER, NULL, NULL},
 	{"default_notation_mode", &(prefs.def_notation), INTEGER, NULL, NULL},
@@ -77,9 +90,9 @@ static s_prefs_entry prefs_list[41] = {
 	{"remembers_display", &(prefs.rem_display), BOOLEAN, "prefs_rem_display", set_checkbutton},
 	{"remembered_valuex", &(prefs.rem_valuex), STRING, NULL, NULL},
 	{"remembered_valuey", &(prefs.rem_valuey), STRING, NULL, NULL},
-	{"remembered_valuez", &(prefs.rem_valuez), STRING, NULL, NULL},
+/*50*/	{"remembered_valuez", &(prefs.rem_valuez), STRING, NULL, NULL},
 	{"remembered_valuet", &(prefs.rem_valuet), STRING, NULL, NULL},
-/*40.*/	{"show_menu_bar", &(prefs.show_menu), BOOLEAN, "prefs_show_menu", set_checkbutton},
+	{"show_menu_bar", &(prefs.show_menu), BOOLEAN, "prefs_show_menu", set_checkbutton},
 	{NULL, NULL, 0, NULL, NULL}
 };
 static char *prefs_list_old_entries[3] = {"show_status_bar", "remembered_value", NULL};
@@ -122,14 +135,26 @@ static void config_file_get_default_prefs (s_preferences *this_prefs)
 	/* constants - handled different */
 	
 	/* 4th pref page */
+	this_prefs->dec_sep = DEFAULT_DEC_SEP;
+	this_prefs->dec_sep_char = g_strdup (DEFAULT_DEC_SEP_CHAR);
+	this_prefs->dec_sep_length = DEFAULT_DEC_SEP_LENGTH;
 	this_prefs->hex_bits = DEFAULT_HEX_BITS;
 	this_prefs->hex_signed = DEFAULT_HEX_SIGNED;
+	this_prefs->hex_sep = DEFAULT_HEX_SEP;
+	this_prefs->hex_sep_char = g_strdup (DEFAULT_HEX_SEP_CHAR);
+	this_prefs->hex_sep_length = DEFAULT_HEX_SEP_LENGTH;
 	this_prefs->oct_bits = DEFAULT_OCT_BITS;
 	this_prefs->oct_signed = DEFAULT_OCT_SIGNED;
+	this_prefs->oct_sep = DEFAULT_OCT_SEP;
+	this_prefs->oct_sep_char = g_strdup (DEFAULT_OCT_SEP_CHAR);
+	this_prefs->oct_sep_length = DEFAULT_OCT_SEP_LENGTH;
 	this_prefs->bin_bits = DEFAULT_BIN_BITS;
 	this_prefs->bin_signed = DEFAULT_BIN_SIGNED;
 	this_prefs->bin_fixed = DEFAULT_BIN_FIXED;
 	this_prefs->bin_length = DEFAULT_BIN_LENGTH;
+	this_prefs->bin_sep = DEFAULT_BIN_SEP;
+	this_prefs->bin_sep_char = g_strdup (DEFAULT_BIN_SEP_CHAR);
+	this_prefs->bin_sep_length = DEFAULT_BIN_SEP_LENGTH;
 
 	/* 5rd pref page */
 	this_prefs->stack_size = DEFAULT_STACK_SIZE;
@@ -214,6 +239,14 @@ void config_file_set_prefs (char *key, char *value)
 		case STRING:
 			string_var = this_var;
 			g_free (*string_var);
+			/* version 1.2.4 introduced strings enbodied by pairs of
+			 * \"'s. So remove those if they exist.
+			 */
+			if ((value[0] == '\"') && (value[strlen(value)-1] == '\"'))
+			{
+				value++;
+				value[strlen(value)-1]='\0';
+			}
 			*string_var = g_strdup (value);
 			break;
 		case BOOLEAN:
@@ -399,7 +432,7 @@ void config_file_write (char *filename, s_preferences this_prefs, s_constant *th
 			switch (prefs_list[counter].key_type) {
 				case STRING:
 					string_var = this_var;
-					line = g_strdup_printf ("%s=%s\n", prefs_list[counter].key, *string_var);
+					line = g_strdup_printf ("%s=\"%s\"\n", prefs_list[counter].key, *string_var);
 					break;
 				case BOOLEAN:
 					bool_var = this_var;
